@@ -36,35 +36,48 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-gTestfile = '15.3.3.1-2.js';
+gTestfile = '15.3.5.1.js';
 
 /**
-   File Name:          15.3.3.1-2.js
-   ECMA Section:       15.3.3.1 Properties of the Function Constructor
-   Function.prototype
+   File Name:          15.3.5.1.js
+   ECMA Section:       Function.length
+   Description:
 
-   Description:        The initial value of Function.prototype is the built-in
-   Function prototype object.
+   The value of the length property is usually an integer that indicates the
+   "typical" number of arguments expected by the function.  However, the
+   language permits the function to be invoked with some other number of
+   arguments. The behavior of a function when invoked on a number of arguments
+   other than the number specified by its length property depends on the function.
 
-   This property shall have the attributes [DontEnum |
-   DontDelete | ReadOnly]
+   this test needs a 1.2 version check.
+
+   http://scopus.mcom.com/bugsplat/show_bug.cgi?id=104204
 
-   This test the DontEnum property of Function.prototype.
 
    Author:             christine@netscape.com
-   Date:               28 october 1997
-
+   Date:               12 november 1997
 */
-var SECTION = "15.3.3.1-2";
+
+var SECTION = "15.3.5.1";
 var VERSION = "ECMA_1";
+var TITLE   = "Function.length";
+var BUGNUMBER="104204";
 startTest();
-var TITLE   = "Function.prototype";
 
 writeHeaderToLog( SECTION + " "+ TITLE);
 
-new TestCase(   SECTION,
-		"var str='';for (prop in Function ) str += prop; str;",
-		"",
-		eval("var str='';for (prop in Function) str += prop; str;")
-  );
+var f = new Function( "a","b", "c", "return f.length");
+
+new TestCase( SECTION,
+	      'var f = new Function( "a","b", "c", "return f.length"); f()',
+	      3,
+	      f() );
+
+
+new TestCase( SECTION,
+	      'var f = new Function( "a","b", "c", "return f.length"); f(1,2,3,4,5)',
+	      3,
+	      f(1,2,3,4,5) );
+
 test();
+
