@@ -16,10 +16,10 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2007
+ * Portions created by the Initial Developer are Copyright (C) 2005
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): moz_bug_r_a4
+ * Contributor(s): Igor Bukanov
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -35,40 +35,39 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-367119-01.js';
+var gTestfile = 'regress-311792-02.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 367119;
-var summary = 'memory corruption in script_exec';
+var BUGNUMBER = 311792;
+var summary = 'Root Array.prototype methods';
 var actual = 'No Crash';
 var expect = 'No Crash';
 
-//-----------------------------------------------------------------------------
-test();
-//-----------------------------------------------------------------------------
-
-function test()
-{
-  enterFunc ('test');
-  printBugNumber(BUGNUMBER);
-  printStatus (summary);
+printBugNumber(BUGNUMBER);
+printStatus (summary);
  
-  if (typeof Script == 'undefined')
-  {
-    print('Test skipped. Script or toSource not defined');
-  }
-  else
-  {
-    var s = new Script("");
-    var o = {
-      valueOf : function() {
-        s.compile("");
-        Array(11).join(Array(11).join(Array(101).join("aaaaa")));
-        return {};
-      }
-    };
-    s.exec(o);
-  }
-  reportCompare(expect, actual, summary);
+var subverted = 0;
 
-  exitFunc ('test');
+function index_getter()
+{
+  delete a[0];
+  gc();
+  for (var i = 0; i != 1 << 14; ++i) {
+    var tmp = new String("test");
+    tmp = null;
+  }
+  return 1;
 }
+
+function index_setter(value)
+{
+  subverted = value;
+}
+
+var a = [ Math.sqrt(2), 0 ];
+a.__defineGetter__(1, index_getter);
+a.__defineSetter__(1, index_setter);
+
+a.reverse();
+printStatus(subverted)
+
+  reportCompare(expect, actual, summary);
