@@ -19,7 +19,7 @@
  * Portions created by the Initial Developer are Copyright (C) 2006
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Seno Aiko
+ * Contributor(s): Pavel Cvrcek [JasnaPaka]
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -35,10 +35,10 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-345879-02.js';
+var gTestfile = 'regress-349023-01.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 345879;
-var summary = 'Crash when calling a function from a generator with less arguments than its arity ';
+var BUGNUMBER = 349023;
+var summary = 'Bogus JSCLASS_IS_EXTENDED in the generator class';
 var actual = 'No Crash';
 var expect = 'No Crash';
 
@@ -54,12 +54,20 @@ function test()
   printStatus (summary);
 
   function gen() {
-    (yield 5)();
-    yield;
+    var i = 0;
+    yield i;
+  }
+
+  try
+  {
+    var g = gen();
+    for (var i = 0; i < 10; i++) {
+      print(g.next());
+    }
+  }
+  catch(ex)
+  {
   }
-  f = gen();
-  f.next();
-  f.send(Function());
  
   reportCompare(expect, actual, summary);
 
