@@ -19,7 +19,7 @@
  * Portions created by the Initial Developer are Copyright (C) 2008
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Brendan Eich
+ * Contributor(s): Jesse Ruderman
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -35,10 +35,10 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-467495-03.js';
+var gTestfile = 'regress-463334-02.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 467495;
-var summary = 'TCF_FUN_CLOSURE_VS_VAR is necessary';
+var BUGNUMBER = 463334;
+var summary = 'TM: Do not crash in isPromoteInt';
 var actual = '';
 var expect = '';
 
@@ -52,29 +52,17 @@ function test()
   enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
+ 
+  jit(true);
 
-  function f(x)
-  {
-    actual = '';
-    var g;
-    print(actual += typeof g + ',');
-
-    if (x)
-      function g(){};
-
-    print(actual += g);
+  for (let i in 
+         (function() { for (let j = 0; j < 4; ++j) with({t: NaN}) yield; })()) 
+  { 
   }
 
-  expect = 'undefined,undefined';
-  f(0);
+  jit(false);
 
-  reportCompare(expect, actual, summary + ': f(0): ');
-
-  expect = 'undefined,function g() {\n}';
-
-  f(1);
-
-  reportCompare(expect, actual, summary + ': f(1): ');
+  reportCompare(expect, actual, summary);
 
   exitFunc ('test');
 }
